# ---------------------------------------------------------------------

# Paper:       	N.M. Tran, M. Osipenko, and W.K. Hrdle

#			"Principal Component Analysis in an Asymmetric Norm"

# ---------------------------------------------------------------------

# Quantlet:    	simcurve.R

# ---------------------------------------------------------------------

# Description: 	Function to simulate the curves with different scenarios
# ---------------------------------------------------------------------

# Usage:       	-
# ---------------------------------------------------------------------

# Inputs:      	see below

# ---------------------------------------------------------------------

# Output:      	see below

# ---------------------------------------------------------------------
# Example:     	-

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------



#--- FUNCTION: simulates curves 
simcurv <- function
###X_ij =mu(t_j)+f_1(t_j)*a_i1+f_2(t_j)*a_i2+...+e_ij, a_i1~N(0,s_1), a_i2~N(0,s_2)
###with option (1) e_ij~ N(0,se), (2) e_ij~t(se), (3)e_ij~N(0,mu(t_j)^2*se), 
### (4)  e_ij~ logN(0,se), (5)  e_ij~ U(0,se)+U(0,se/2)
###Returns vectorized X and X_e theoretical expectile of X stored in a (2pn+4)xK*length(tauseq).
###Rows: vectorized X and X_e resp.+ 4 parameters: tau level,scenario number,n,p. Columns: K simulation runs for each tau in tauseq.
###The default option returns X with the normal error.
(mu, 
###mean curve, px1, p is the data dimension
f, 
###pxk for k principal directions
option = 1, 
###option for the error distribution
s,
#c(s1, s2,.., se) parameters of the distributions for a and e
tauseq,
###the expectile level, 
###s1,s2,se are variances for normal distributions, se are degrees of freedom for t distr.
N,P,K
### dimensions of the data: N number of curves, P grid, K number of simulation runs
){
	set.seed(1234567)
	k=length(s)-1
	se=s[k+1]
	mua=array(mu, c(P,N,K)); fa=array(NA, c(P,N,K))
	a=sweep(array(rnorm(k*N*K),c(k,N,K)),MARGIN=1,sqrt(s[1:k]),`*`)
	output=matrix(NA,(2*N*P+4),length(tauseq)*K)
	for (y in 1:length(tauseq)){
		set.seed(123)
		tau=tauseq[y]
		X  = array(NA,c(P,N,K))
		X_e= array(NA,c(P,N,K)) #the individual expectile
		if (option==1){
			r=array(rnorm(N*P*K),c(P,N,K))
			e=sqrt(se)*r
			etau=array(enorm(tau, sd=sqrt(se)), c(P,N,K))
		} else if (option==2) {
			e=array(rt(N*P*K,se),c(P,N,K))
			etau=array(et(tau, se), c(P,N,K))
		} else if (option==3) {
			r=array(rnorm(N*P*K),c(P,N,K))
			e=sweep(r,MARGIN=1,sqrt(se*mu),`*`)
			etau=numeric(P)
			for (i in 1:P){
				etau[i]=enorm(tau, sd=sqrt(se*mu[i]))
			}
			etau=array(etau,c(P,N,K))
		} else if (option==4) {
			e=rlnorm(N*P*K,sdlog=sqrt(se)); e=array(e,c(P,N,K))
			etau=array(elnorm(tau, sdlog=sqrt(se)), c(P,N,K))
		} else if (option==5) {
			ee=runif(100000,max=se)+runif(100000,max=se)
			e=runif(N*P*K,max=se)+runif(N*P*K,max=se); e=array(e,c(P,N,K))
			etau=array(expectile(matrix(ee,1,100000),alpha=(tau-0.5)),c(P,N,K))
		} else {stop("'option' takes integer values 1 to 5")}	
		for (j in 1:K){
			fa[,,j]=f%*%a[,,j]
		}
		X  = mua+fa+e
		X_e= mua+fa+etau
		output[,(K*(y-1)+1):(K*y)]=rbind(matrix(X,N*P,K),matrix(X_e,N*P,K),rep(tau,times=K),rep(option,times=K),rep(n,times=K),rep(p,times=K))
	}
	return(output)
}
