# ---------------------------------------------------------------------

# Paper:       	N.M. Tran, M. Osipenko, and W.K. Hrdle

#			"Principal Component Analysis in an Asymmetric Norm"

# ---------------------------------------------------------------------

# Quantlet:    	simulation_setup

# ---------------------------------------------------------------------

# Description: 	This is the simulation to attain finite sample 

#			properties of of TopDown, BottomUp and PrincipalExpectile 

#			algorithms 
# ---------------------------------------------------------------------

# Usage:       	depends on function libraries fpca2.R, 

#			PrincipalDirection2.R, simcurve2.R,algeval.R, simeval3.R

# ---------------------------------------------------------------------

# Inputs:      	params.txt file containing parameters of the simulation setup

# ---------------------------------------------------------------------

# Output:      	results.RData

# ---------------------------------------------------------------------

# Example:     	Estimation is done for the expectile levels 0.90, 

#			0.95, and 0.975.

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------

rm(list = ls())
graphics.off()

mainDir <- "H:/Asymmetric norm/Rcodes/PCA_in_an_asymmetric_norm/simulation" #enter your main directory
#install.packages("fda")
#install.packages("expectreg")
library("fda")
library("expectreg")
source(file.path(mainDir,"fpca2.R"))
source(file.path(mainDir,"simcurve2.R"))
source(file.path(mainDir,"simeval3.R")) 
source(file.path(mainDir,"algeval.R"))
source(file.path(mainDir,"principalDirection2.R"))

#------------------parameters
params<-read.table(file.path(mainDir, "params.txt"))# file contain parameters of the simulation (sample size, taus)
K=500 #number of simulation turns
#run with 2 components
nc = 2
noptions=5 #number of scenarios
#optim params
tol = 1e-10
max.iter=30
#----------------mean and component functions
m<-function(t){(mu=1+t+exp(-(t-0.6)^2/0.05))
		return(mu)
}
ff<-function(t){f1=sqrt(2)*sin(2*pi*t)
		f2=sqrt(2)*cos(2*pi*t)	
		return(cbind(f1,f2))
}

#--------------------taus
tauseq=c(0.9,0.95,0.975)

#----------------generate the curves (takes some time: 15 min)
XX=as.list(1:(nrow(params)))
for (q in 1:nrow(params)){
	print(q)
	n=params[q,1]; p=params[q,2]
	#variance parameter of N(0,s)	
	t=seq(0,1,length=p)
	s1=params[q,3]
	s2=params[q,4] 
	se=params[q,5]
	s=c(s1,s2,se)
	#degrees of freedom for t
	df=params[q,6]
	mu=m(t); f=ff(t)
	X=NULL
	for (h in 1:noptions){
		option=h; s[3]=ifelse(option==2,df,se)
		X=cbind(X,simcurv(mu=mu,f=f,option=h,s=s,tauseq=tauseq, N=n,P=p,K=K))#output matrix 2np x length(tauseq)K
		}
		XX[[q]]<-X
}
#-----------------compute the simulation statistics (takes around 5 days to compute)
results<-lapply(XX,simeval)
#save(results,file=paste0("results",".RData"))

#------------------------sort and save the results
lth=c(nrow(results[[1]]),nrow(results[[2]]),nrow(results[[3]]),nrow(results[[4]]),nrow(results[[5]]),nrow(results[[6]]))
#extract results for each setting (setting 1: amse1 for tau=0.9, amse2 for tau=0.95
#amse3 for tau=0.975; setting 2: amse4 for tau=0.9, amse5 for tau=0.95, and amse6 for tau=0.975):
write.table(results[[1]][c((lth[1]-15):lth[1]),],"amse1.txt")
write.table(results[[2]][c((lth[2]-15):lth[2]),],"amse2.txt")
write.table(results[[3]][c((lth[3]-15):lth[3]),],"amse3.txt")
write.table(results[[4]][c((lth[4]-15):lth[4]),],"amse4.txt")
write.table(results[[5]][c((lth[5]-15):lth[5]),],"amse5.txt")
write.table(results[[6]][c((lth[6]-15):lth[6]),],"amse6.txt")

#extract bases for small sample and tau=0.95
p=100; n=20
from=501; to=1000 #scenario 1 tau=0.95
#basis#--------setting 1
write.table(results[[1]][1:(2*p),from:to],"basis_bup.txt")
write.table(results[[1]][(2*p+1):(4*p),from:to],"basis_td.txt")
write.table(results[[1]][(4*p+1):(6*p),from:to],"basis_pec.txt")
#basis#--------setting 2
write.table(results[[4]][1:(2*p),from:to],"basis_bup_2.txt")
write.table(results[[4]][(2*p+1):(4*p),from:to],"basis_td_2.txt")
write.table(results[[4]][(4*p+1):(6*p),from:to],"basis_pec_2.txt")





