# ---------------------------------------------------------------------

# Paper:       	N.M. Tran, M. Osipenko, and W.K. Hrdle

#			"Principal Component Analysis in an Asymmetric Norm"

# ---------------------------------------------------------------------

# Quantlet:    	simulation_summary

# ---------------------------------------------------------------------

# Description: 	This is the summary of the simulation to attain finite sample 

#			properties of of TopDown, BottomUp and PrincipalExpectile 

#			algorithms 
# ---------------------------------------------------------------------

# Usage:       	depends on function libraries fpca2.R, 

#			PrincipalDirection2.R, simcurve3.R,algeval.R, simeval.R

# ---------------------------------------------------------------------

# Inputs:      	parameters.txt

# ---------------------------------------------------------------------

# Output:      	Plots and tables shown in the paper

# ---------------------------------------------------------------------

# Example:     	Estimation is done for the expectile levels 0.90, 

#			0.95, and 0.975.

# ---------------------------------------------------------------------

# Author:      	Maria Osipenko

# ---------------------------------------------------------------------
rm(list = ls())

mainDir <- "H:/Asymmetric norm/Rcodes/PCA_in_an_asymmetric_norm/simulation" #enter your main directory
subDir <- format(Sys.Date(),format="%Y%m%d") #subdirectory named after the system date
setwd(file.path(mainDir))
#figDir <- file.path(mainDir,"figures")#directory for figures



#------------------------sorting the results
amse1 = read.table("amse1.txt")
amse2 = read.table("amse2.txt")
amse3 = read.table("amse3.txt")
amse4 = read.table("amse4.txt")
amse5 = read.table("amse5.txt")
amse6 = read.table("amse6.txt")

#-------------------------simulation setup 1: MSE (Table III)---------------------------------
#-------------------------------------------------------------------------------------------
amse_table=data.frame(rbind(t(amse1),t(amse2),t(amse3)))
colnames(amse_table)=c("BUP","TD","PEC","time_BUP","time_TD","time_PEC","conv_BUP","conv_TD",
"conv_PEC","RSS_BUP","RSS_TD","RSS_PEC","tau","scen","n","p")
RSS_TD=amse_table$RSS_TD
RSS_BUP=amse_table$RSS_BUP
amse_table[which(RSS_BUP>3*RSS_TD),c(1,4,7,10)]=NA # detect false convergencies and set to NA
amse_table[which(RSS_TD>3*RSS_BUP),c(2,5,8,11)]=NA

tauseq=c(0.9,0.95,0.975)
scen=seq(1,5)
params<-cbind(rep(scen,each=length(tauseq)),rep(tauseq,times=length(scen)))
nseq=c(20,50,100)

stats_table<-function(amse_table,params,nseq){
	out2=NULL
	for (h in 1:nrow(params)){
		out1=NULL
		for (r in 1:length(nseq)){
			tau=params[h,2]; scen=params[h,1]
			sample=nseq[r]
			ind=which(amse_table[,14]==scen&amse_table[,13]==tau&amse_table[,15]==sample)
			out<-rbind(apply(amse_table[ind,1:3],2,mean,na.rm=T),apply(amse_table[ind,1:3],2,sd,na.rm=T))
			out1=cbind(out1,out)
		}
		out2=rbind(out2,out1)
	}
	colnames(out2)<-c("BUP_small","TD_small","PEC_small","BUP_medium","TD_medium","PEC_medium","BUP_large","TD_large","PEC_large")

	return(out2)
}
labels=cbind(rep(1:5,each=6),rep(rep(c(0.9,0.95,0.975),each=2),times=5),rep(c("mean","sd"),times=15))
colnames(labels)=c("scenario","tau","statistics")
print("Table III")
(big_table=data.frame(labels,round(stats_table(amse_table,params,nseq),4)))

#----------------------------simulation setup 2: MSE (Table IV)---------------------------
#----------------------------------------------------------------------------------------
amse_table_2=data.frame(rbind(t(amse4),t(amse5),t(amse6)))
colnames(amse_table_2)=c("BUP","TD","PEC","time_BUP","time_TD","time_PEC","conv_BUP","conv_TD",
"conv_PEC","RSS_BUP","RSS_TD","RSS_PEC","tau","scen","n","p")
RSS_TD=amse_table_2$RSS_TD
RSS_BUP=amse_table_2$RSS_BUP
amse_table_2[which(RSS_BUP>3*RSS_TD),c(1,4,7,10)]=NA # set false convergencies to NA
amse_table_2[which(RSS_TD>3*RSS_BUP),c(2,5,8,11)]=NA
print("Table IV")
(big_table_2=data.frame(labels,round(stats_table(amse_table_2,params,nseq),4)))


#---------------------------computational time (Table I)---------------------------------
#----------------------------------------------------------------------------------------
conv<-amse_table[,7:9]
ctime.bup<-tapply(amse_table$time_BUP,list(amse_table$tau,amse_table$n),mean,na.rm=TRUE)
ctime.td<-tapply(amse_table$time_TD,list(amse_table$tau,amse_table$n),mean,na.rm=TRUE)
ctime.pec<-tapply(amse_table$time_PEC,list(amse_table$tau,amse_table$n),mean,na.rm=TRUE)

ctime=round(data.frame(ctime.bup[,1],ctime.td[,1],ctime.pec[,1],
ctime.bup[,2],ctime.td[,2],ctime.pec[,2],
ctime.bup[,3],ctime.td[,3],ctime.pec[,3]),2)

colnames(ctime)=c("BUP_small","TD_small","PEC_small","BUP_medium","TD_medium","PEC_medium","BUP_large","TD_large","PEC_large")
print("Table I")
ctime

#----------------------------convergence ratio (Table II)-----------------------------------
#------------------------------------------------------------------------------------------
sumna<-function(vec){
	return(sum(is.na(vec))/length(vec))	
}

conv.bup<-tapply(amse_table$BUP,list(amse_table$tau,amse_table$scen,amse_table$n),sumna)
conv.td<-tapply(amse_table$TD,list(amse_table$tau,amse_table$scen,amse_table$n),sumna)
conv.pec<-tapply(amse_table$PEC,list(amse_table$tau,amse_table$scen,amse_table$n),sumna)

conv.bup<-tapply(amse_table$BUP,list(amse_table$tau,amse_table$n),sumna)
conv.td<-tapply(amse_table$TD,list(amse_table$tau,amse_table$n),sumna)
conv.pec<-tapply(amse_table$PEC,list(amse_table$tau,amse_table$n),sumna)

conv=round(data.frame(conv.bup[,1],conv.td[,1],conv.pec[,1],
conv.bup[,2],conv.td[,2],conv.pec[,2],
conv.bup[,3],conv.td[,3],conv.pec[,3]),2)

colnames(conv)=c("BUP_small","TD_small","PEC_small","BUP_medium","TD_medium","PEC_medium","BUP_large","TD_large","PEC_large")
print("Table II")
conv

#--------------------------plot the estimated pcs
p=100; n=20
from=501; to=1000 #scenario 1 tau=0.95

#basis#--------ratio 1
basis.bup= read.table("basis_bup.txt")
basis.td = read.table("basis_td.txt")
basis.pec= read.table("basis_pec.txt")

#mean of bases
meanbasis.bup=apply(basis.bup[,which(amse_table$conv_BUP[from:to]==1)],1,mean,na.rm=T)
meanbasis.td =apply(basis.td[,which(amse_table$conv_TD[from:to]==1)],1,mean,na.rm=T)
meanbasis.pec=apply(basis.pec[,which(amse_table$conv_PEC[from:to]==1)],1,mean,na.rm=T)

expgr<-function(n){
		col="gray88"
		return(col)
}

plot.vec <- function(mat, main.str = "",type="l",pch=1,add=FALSE,ylim=NA,lwd=2,color=rainbow,colp="gray80",xlab="",ylab="")
{
	a=1
	if(any(is.na(ylim))){ylim=c(min(mat),max(mat))}
	if (add==FALSE){ 
    		a=2	
		plot(mat[,1], type = type, lwd=lwd, ylim = ylim, main = main.str, pch=pch,col=colp,axes=FALSE,xlab=xlab,ylab=ylab)
    	}
	for(i in c(a:dim(mat)[2])){
		if (type=="p"){
		points(mat[,i], col = colp,pch=pch)
		} else { lines(mat[,i], col = color(i),lwd=lwd)}
    } 
}
ff<-function(t){f1=sqrt(2)*sin(2*pi*t)
		f2=sqrt(2)*cos(2*pi*t)	
		return(cbind(f1,f2))
}
#------------------------------Figure 1---------------------------------------------------------
#-----------------------------------------------------------------------------------------------
# 1st component BUP, setup 1
Figure_1<-X11()
par(mfrow=c(3,2))
#pdf(file = file.path(figDir,"BUP1.pdf"),paper="special",height=6,width=6)
plot.vec(basis.bup[1:100,which(amse_table$conv_BUP[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.bup[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component BUP, setup 1
#pdf(file = file.path(figDir,"BUP2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.bup[101:200,which(amse_table$conv_BUP[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.bup[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()

# 1st component TD, setup 1
#pdf(file = file.path(figDir,"TD1.pdf"),paper="special",height=6,width=6)
plot.vec(basis.td[1:100,which(amse_table$conv_TD[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.td[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component TD, setup 1
#pdf(file = file.path(figDir,"TD2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.td[101:200,which(amse_table$conv_TD[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.td[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()

# 1st component PEC, setup 1
#pdf(file = file.path(figDir,"PEC1.pdf"),paper="special",height=6,width=6)
plot.vec(basis.pec[1:100,which(amse_table$conv_PEC[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.pec[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component PEC, setup 1
#pdf(file = file.path(figDir,"PEC2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.pec[101:200,which(amse_table$conv_PEC[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.pec[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()

#------------------------------Figure 2---------------------------------------------------------
#-----------------------------------------------------------------------------------------------

basis.bup = read.table("basis_bup_2.txt")
basis.td  = read.table("basis_td_2.txt")
basis.pec = read.table("basis_pec_2.txt")

#mean of basises
meanbasis.bup= apply(basis.bup[,which(amse_table_2$conv_BUP[from:to]==1)],1,mean,na.rm=T)
meanbasis.td = apply(basis.td[,which(amse_table_2$conv_TD[from:to]==1)],1,mean,na.rm=T)
meanbasis.pec= apply(basis.pec[,which(amse_table_2$conv_PEC[from:to]==1)],1,mean,na.rm=T)

# 1st component BUP, setup 2
Figure_2<-X11()
par(mfrow=c(3,2))

#pdf(file = file.path(figDir,"BUP1_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.bup[1:100,which(amse_table_2$conv_BUP[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.bup[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component BUP, setup 2
#pdf(file = file.path(figDir,"BUP2_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.bup[101:200,which(amse_table_2$conv_BUP[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.bup[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.25,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()

# 1st component TD, setup 2
#pdf(file = file.path(figDir,"TD1_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.td[1:100,which(amse_table_2$conv_TD[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.td[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component TD, setup 2
#pdf(file = file.path(figDir,"TD2_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.td[101:200,which(amse_table_2$conv_TD[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.td[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()

# 1st component PEC, setup 2
#pdf(file = file.path(figDir,"PEC1_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.pec[1:100,which(amse_table_2$conv_PEC[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.pec[1:100],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,1],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()
# 2nd component PEC, setup 2
#pdf(file = file.path(figDir,"PEC2_2.pdf"),paper="special",height=6,width=6)
plot.vec(basis.pec[101:200,which(amse_table_2$conv_PEC[from:to]==1)],ylim=c(-2,2),type="l",color=expgr)
lines(meanbasis.pec[101:200],lwd=2,lty=2)
lines(ff(seq(0,1,length=100))[,2],lwd=2)
axis(2, lwd=1,cex.axis=1.3,at=seq(-2,2,by=1))
axis(1, lwd=1,cex.axis=1.3,at=seq(1,100,length=3),labels=seq(0,1,length=3))
#dev.off()





